@@ -10,6 +10,7 @@ import os
 from pathlib import Path
 import pytest
 import time
+import sys
 import yaml
 
 from .util import (
@@ -1026,11 +1027,9 @@ class TestConfig:
         assert expected_log_record in caplog.record_tuples
 
     def test_load_invalid(self, tmp_path):
-        with pytest.raises(receive.OTReceiveConfigError) as excinfo:
+        # Passing a non-path as the config file should fail.
+        with pytest.raises(receive.OTReceiveConfigError, match='PathLike'):
             receive.OTReceiveConfig.load([True])
-        assert str(excinfo.value) == (
-            'expected str, bytes or os.PathLike object, not bool'
-        )
 
         path = tmp_path / 'ostree-receive.conf'
         data = {
@@ -1311,3 +1310,31 @@ class TestArgParser:
         assert args.log_level == 'WARNING'
         args = ap.parse_args(['--quiet', 'repo', 'url'])
         assert args.log_level == 'WARNING'
+
+
+def test_compat_main(monkeypatch):
+    """Check compat_main dispatching"""
+    from otpush import receive_legacy
+
+    monkeypatch.setattr(receive, 'main', lambda: 'current')
+    monkeypatch.setattr(receive_legacy, 'main', lambda: 'legacy')
+
+    with monkeypatch.context() as ctx:
+        ctx.setattr(sys, 'argv', ['ostree-receive'])
+        assert receive.compat_main() == 'current'
+
+    with monkeypatch.context() as ctx:
+        ctx.setattr(sys, 'argv', ['ostree-receive', '--opt'])
+        assert receive.compat_main() == 'current'
+
+    with monkeypatch.context() as ctx:
+        ctx.setattr(sys, 'argv', ['ostree-receive', '--opt', 'repo'])
+        assert receive.compat_main() == 'current'
+
+    with monkeypatch.context() as ctx:
+        ctx.setattr(sys, 'argv', ['ostree-receive', '--repo=repo'])
+        assert receive.compat_main() == 'legacy'
+
+    with monkeypatch.context() as ctx:
+        ctx.setattr(sys, 'argv', ['ostree-receive', '--opt', '--repo=repo'])
+        assert receive.compat_main() == 'legacy'
